
// acces.h
// Definition des classes permettant l'algorithme d'accessibilit  la vole
//
// Agnes, 25 Juillet 2001
//


#ifndef ACCES_H
#define ACCES_H

#include "liste.h"
#include "zone.h"
#include "ta.h"
#include "lnu.h"



//-----------------------------------------------
//  Accessibilite simple - dans un AT
//-----------------------------------------------


void Reachability(TA * A,Ident n);


// Rconfig = un noeud + une zone

class Rconfig		
{
private:
	Ident _noeud;
	Zone * _zone;
	TA * _automate;
 public:
 	Rconfig()
	  {_noeud=0;_zone=0;_automate=0;}
	Rconfig(Ident n,Zone * z, TA * aut)
	  {_noeud=n;_zone=z;_automate=aut;}
	void aff();
	Ident n();
	int Phi();		// renvoie un numero associe au noeud
	Liste<Rconfig> Succ();

	friend int operator == (Rconfig c1,Rconfig c2);
	friend int operator <= (Rconfig c1,Rconfig c2);
};

int operator == (Rconfig c1,Rconfig c2);
int operator <= (Rconfig c1,Rconfig c2);


// PSet = structure pour stocker les Rconfig deja vues,
// on les repartit en plusieurs listes selon le numero Phi().
class PSet {		
 private:

  class elt {	
    public:
    Rconfig couple;
    elt *suivant;
    elt()
      {suivant=0;}
    elt(Rconfig c, elt* e)
      {couple=c;suivant=e;}
  };

  elt * *_liste;

 public:
  PSet(int taille);
  void aff(int taille);
  int contient(Rconfig c);
  void ajoute(Rconfig c);
};


// WSet = structure FIFO pour stocker les Rconfig non testees
class WSet {
private:

  class elt {
    public:
    Rconfig couple;
    elt * suivant;
    elt()
      {suivant=0;}
    elt(Rconfig c, elt* e)
      {couple=c;suivant=e;}
   };

  elt * _dernier;

 public:
  WSet();
  WSet(Rconfig c);
  void aff();
  int est_vide();
  Rconfig piocher();
  void fusionne(Liste<Rconfig> l);
  int appartient(Rconfig c);
  void ajoute (Rconfig c);
};




//---------------------------------------------------
// Accessibilite dans les systemes paralleles
//---------------------------------------------------


class RProblem {
private:
  	Lnu * _goal;
  	Ident * * _Table;
  	TA * * _Aut;
  	Ident * _NomAut;
  	RegionGraph * _RG;
  	int _Tab;
  	int _NbVec;

public:
  	RProblem(STable * T,Lnu * prop);

  	Lnu * Prop()
	  {return _goal;}
  	RegionGraph * RG()
	  {return _RG;}
	int NbVec() {return _NbVec;};
	int NbAut() {return _Tab;};
	Ident Table(int i, int j)
	  {return _Table[i][j];};
	Ident NomAut(int i)
	  {assert(i<_Tab);return _NomAut[i];};
	TA * Aut(int i)
	  {return _Aut[i];};
	
  	void Print();
  	int NumAut(Ident nom);
  	void Reach();
};



class RSconfig		// couple d'un vecteur de noeuds et d'une zone temporelle
{
private:
  Ident * _noeud;
  Zone * _zone;
  int _Tab;
  RProblem * _RP;
public:
  static int NbVisitSucc;
  RSconfig()
    {_noeud=0;_zone=0;_Tab=0;_RP=0;}
  RSconfig(RProblem * RP);
  RSconfig(RSconfig * c,Pt_Transition * t,Zone * z);
  RProblem * RP()
    {return _RP;}
  Ident Etat(int i)
    {assert(i<_Tab);return _noeud[i];}
  Zone * ZoneC()
    {return _zone;}
  int Phi();
  Liste<Pt_Transition *> SuccV(int i);
  Liste<Pt_Transition *> SuccT();
  Liste<RSconfig> Succ();
  int Verify(Ident nta,Ident prop);
  void Print();

   friend int operator == (RSconfig c1,RSconfig c2);
   friend int operator >= (RSconfig c1,RSconfig c2);
};

//int RSconfig::NbVisitSucc=0;

int operator == (RSconfig c1,RSconfig c2);
int operator >= (RSconfig c1,RSconfig c2);



// structure pour stocker les Rconfig deja vues,
// en les classant par numero de noeuds
class P_Set {	
 private:

  class elt {
  public:
    RSconfig couple;
    elt *suivant;
    elt()
      {suivant=0;}
    elt(RSconfig c, elt* e)
      {couple=c;suivant=e;}
  };

  elt * *_liste;
  int _nbliste;
 public:
  P_Set(int taille);
  void aff(int taille);
  int contient(RSconfig c);
  void ajoute(RSconfig c);
  int Taille();
};


// W_Set = structure FIFO pour stocker les Rconfig non testees
class W_Set {
 private:
  class elt { 
    public :
      RSconfig couple;
    elt * suivant;
  };
  
  elt * _dernier;

 public:
  W_Set();
  W_Set(RSconfig c);
  void aff();
  int est_vide();
  RSconfig piocher();
  void fusionne(Liste<RSconfig> l);
  int appartient(RSconfig c);
  void ajoute (RSconfig c);
  int Taille();
};

#endif

