/* File: env.h -- Francois -- Last modified on 2 Feb 2001
 * 	
 *      classes for environment
 *
 */


#ifndef ENV_H
#define ENV_H


#include "liste.h"
#include "map.h"
#include "type.h"
#include "arbre.h"
#include "lnu.h"
#include "specif.h"
#include "ta.h"
#include "table.h"

class Specif;
typedef Specif * Pt_Specif;

typedef Mapping<Pt_Specif> Map_spec;
typedef Mapping<Pt_Declaration> Map_decl;
typedef Mapping<Pt_Lnu> Map_Lnu;
typedef Mapping<Pt_TA> Map_ta;
typedef Mapping<Pt_Fsynchro> Map_fs;
typedef Mapping<int> Map_param;

class ASNoeud;
class Objet;
typedef Objet * PtObjet;
typedef Liste<Objet *> LObjet;
typedef PtObjet (*TFCT)(LObjet *);
//typedef PtObjet (*) (LObjet *) TFCT;

class Objet {
  private :
    type _ot;
    char * _envname;
  union {
    int _val;
    Lnu * _form;
    TA * _ta;
    Declaration * _d;
    Specif * _spec;
    Fsynchro * _fs;
    TFCT _fct;
    char * _vname;
    STable * _stab;
    Liste<VerifElet *> * _syst;
    Zone * _zone;
    SetVal * _setv;
    Ident_L *_lid;
  };
  public :

    Objet(type t,int v,char * nom=0);  // nom correspond au nom ds l'envir
    Objet(Lnu * v,char * nom=0);
    Objet(TA * v,char * nom=0);
    Objet(Declaration * v,char * nom=0);
    Objet(Specif * v,char * nom=0);
    Objet(Fsynchro * v,char * nom=0);
    Objet(type t,TFCT v,char * nom=0);
    Objet(type t,char * v,char * nom=0);
    Objet(STable * tab,char * nom =0);
    Objet(Liste<VerifElet *> * s,char * nom =0);
    Objet(Zone * z,char * nom =0);
    Objet(SetVal * z,char * nom =0);
    Objet(Ident_L * l,char * nom =0);

    void Print(int jtype=0); // jtype =1 pour n'afficher que le type des objets
    void Destroy();
    Objet * Copy();

    type Otype();
    char * OEnvname();
    void PutEnvname(char * nom);
    Lnu * OLnu();
    TA * OTA();
    Declaration * ODecl();
    Specif * OSpec();
    Fsynchro * OFsyn();
    TFCT Ofct();
    char * OVname();
    int Oval();
    STable * Otable();
    Liste<VerifElet *> * Osystem();
    Zone * Ozone();
    SetVal * Osetval();
    Ident_L * Olid();
};


class TEval {
  public :
  int flag;
  Objet * res;
};

typedef Liste<Objet *> LObjet;
typedef Objet * Pt_Objet;

class Env {
private:
  
  class Element {
  public:
    Objet * _val;
    Element * _suivant;
    //    void Destroy();
  };
  typedef Element * Ptr_Element;

  Ptr_Element debut;
  void initialisation();
 public:

  Env()
    {debut = 0;initialisation();}

  void Ajouter(char * n, Objet * x);
  void Ajouterf(char * n,Objet * x);
  void Supprimer(char * n);
  Objet * Recherche(char * n,int cs =0);
  int Est_def(char * n,int cs =0); // cs =1 si case sens ou 0 sinon
  int Est_def(char * n, Pt_Objet & po, int cs =0);
  int Taille();
  int Est_vide();
  void Vide();
  void Print();

  Objet * Interpreter(char * n,LObjet * la, int com =0);
  TEval Appliquer(char * n,LObjet * la);
  TEval Eval(ASNoeud * n,int toplevel =0);
  int VarNonDef(char * t, char * n); // l'objet n de type t n'est pas def... 0 est retourne
};



extern Env Envir;
extern int SIMPLEE;
extern int SIMPLTE;
extern int SIMPLHZ;
extern int SIMPLHZ2;
extern int SIMPLSPLIT;
extern int SIMPLUNGVAR;
extern int SIMPLSPLITaux;
extern int PTRACE;
extern int VERBOSE;
extern char * LATEXOUTPUT;
extern char * VCGOUTPUT;
extern int BELL;

extern int __HACHAGE;

extern int __DBMPLC;

extern int __FIXPOINT;
extern int __NBFIXPOINT;  
extern int __MINFIXPOINT;  
extern int __HYBRID;     
extern int __CMAX; 
extern int __ABSTRACTION;
extern int __OVERAPPROX;
extern int __UNDERAPPROX;

 
extern int __CPLNU;
extern int __CPSPEC;
extern int __CPZONE;
extern int __CPSR;
extern int __CPCF;
extern int __OPTIZONE;
//extern int __OPTIEXTRAIRE;
// extern int __OPTIFORALL1;
// extern int __OPTIFORALL2;
extern int __OPTICOMPACT;
extern int __NOPTICOMPACT;
extern int __OPTIEC;
extern int __NOPTIEC;
extern int __NOUVELLESPEC;
extern int * __TABRESQUOT[2];
extern int __LASTQUOTIENT;


extern int yylinenumber;   

#endif

