/* File: zone.h -- Francois -- Last modified on 2 Feb 2001
 * 	
 *      Zones et DBM
 *
 */

#include "liste.h"
#include "map.h"
#include "lnu.h"

#include "specif.h"
#include "ta.h"
#include "env.h"


#ifndef ZONE_H
#define ZONE_H

class Borne {
  private :
    int _val;
    int _epsi;  // 0: strict <, 1: non strict <=, -1: infini 
  public :
    Borne();  // <= infty
    Borne(int v); // version non stricte par defaut: Borne(0) = "<=0"
    Borne(int v,int e);

    Borne Reverse();
    void ImposeNonStricte();
    Borne Increm(int k,int M);
    void Reduction(int M);
    int Is_Inf();
    int Is_Supe0();
    void Print();
    void PrintRev();
    int Val();

  friend int operator <= (const Borne &, const Borne &);
  friend int operator == (const Borne &, const Borne &);
  friend int operator != (const Borne &, const Borne &);
  friend int operator < (const Borne &, const Borne &);
  friend Borne min (const Borne &, const Borne &);
  friend Borne operator + (const Borne &, const Borne &);
  friend Borne operator + (const Borne &, int );
  friend ostream & operator << (ostream &, const Borne &);
  
  friend int Est_egal(const Borne &,const Borne &);

};

int operator <= (const Borne & b1, const Borne & b2);
int operator == (const Borne & b1, const Borne & b2);
int operator != (const Borne & b1, const Borne & b2);
int operator < (const Borne & b1, const Borne & b2);
Borne min (const Borne & b1, const Borne & b2);
Borne max (const Borne & b1, const Borne & b2);
Borne operator + (const Borne & b1, const Borne & b2);
Borne operator + (const Borne & b, int k);
ostream & operator << (ostream &, const Borne &);
int Est_egal(const Borne &,const Borne &);

class RegionGraph {
  private :
    int N;               // nb d'horloges
    int _Max;           // Cste Max apparaissant dans la spec.
    int _hybrid;        // 1 ssi spec hybride
    Ident * Name;        // Nom de chaque horloge - taille N 
    Mapping<int> * Code; // Numero de chaque horloge
  public :
    RegionGraph(Ident_L l,int mc,int ishyb);
    RegionGraph(RegionGraph * r1, RegionGraph * r2); 

    int Contient(RegionGraph * r);

    int Taille();
    Ident NameC(int i);
    int CodeC(Ident n);
    int MaxC();
    int Is_hybrid();
    void Print();
    void Destroy();
};

class Borne;
typedef Borne * Pt_Borne;

class Zone {
  private :
    int _est_vide;
    Borne * * _Dbm;
    RegionGraph * _rg;    // RegionGraph correspondant
    int _Ndbm;            // nb d'horloges (incluant x0) : Taille(rg)+1

  public :
    Zone();
    Zone(RegionGraph * r,int Vide = 0);
    Zone(RegionGraph * r,Borne b,int vi,int vj);  // ztot + _D[vi][vj]=b
    Zone(RegionGraph * r,LnuTest * f);

    void Print();
    void PrintShort();
    void PrintM();
    void Floyd();
    void Normal();

    void ExtensionRG(RegionGraph * r);

    void Compact();  
    void Destroy() ; 
    void Destroy_D() ; 
    Zone * Copy();
    void Initial();
    RegionGraph * Get_RegionG();
    int Est_vide();
    void Vider();

    void Intersection(Zone * z);
    Zone * IntersectionC(Zone * z);
    void ApproxUnion(Zone * z);   // prend l'enveloppe convexe de this+z
    void Add_Past(Ident_L *ls=0,int cforall =0); // cforal=1 si appel de Forall
    Zone * Add_PastC(Ident_L *ls=0);
    int Est_NonBorne(Ident_L *ls); //  1 ssi ttes les horloges actives ont l'infi. 
    void Forall(Ident_L *ls=0);
    void Add_Future(Ident_L *ls=0);  
    void Add_FermetureTemp(Ident_L *ls=0);  
    Zone * Add_FutureC(Ident_L *ls=0);

    void Projection(Ident c, Ident ca =0, int ia =0);
    void Rev_In(Ident c, Ident ca =0, int ia =0);     
    Zone * Rev_InC(Ident c, Ident ca =0, int ia =0);

    SetReg * Complement();
    SetReg * Extraire(Zone * z);

    int Est_tot();
    int Est_inclu(Zone * z);   // test si *z est inclu dans *this
    int Est_equiv(Zone * z);   // test si *z = *this

};


typedef Zone * Pt_Zone;

extern int __ZADDFUTUR;
extern int __ZADDPAST;
extern int __ZREVIN;
extern int __ZPROJ;
extern int __ZINTER;
extern int __ZFORAL;
extern int __ZESTINCL;
extern int __ZCOMPL;
extern int __ZCOMPLOPTI;
extern int __ZNORMAL;
extern int __NOABSTRACT;

#endif

